"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebServer = void 0;
const koa_1 = __importDefault(require("koa"));
const koa_logger_1 = __importDefault(require("koa-logger"));
const cors_1 = __importDefault(require("@koa/cors"));
const koa_body_1 = __importDefault(require("koa-body"));
const koa_static_1 = __importDefault(require("koa-static"));
const internal_ip_1 = require("internal-ip");
const open_1 = __importDefault(require("open"));
const path_1 = require("path");
const child_process_1 = require("child_process");
const node_toolkit_1 = require("@tech_query/node-toolkit");
const ProxyAgent_1 = require("./ProxyAgent");
__exportStar(require("./ProxyAgent"), exports);
class WebServer {
    constructor(_a = {}) {
        var { proxyMap = {} } = _a, rest = __rest(_a, ["proxyMap"]);
        Object.assign(this, rest);
        this.address = null;
        this.core = new koa_1.default();
        this.proxyMap = (0, node_toolkit_1.patternOf)(proxyMap);
        this.boot();
    }
    boot() {
        this.core.use((0, koa_logger_1.default)());
        if (this.proxyMap)
            this.core
                .use((0, koa_body_1.default)({ multipart: true }))
                .use((0, ProxyAgent_1.ProxyAgent)(this.proxyMap));
        if (this.XDomain)
            this.core.use((0, cors_1.default)());
        this.core.use((0, koa_static_1.default)(this.staticPath));
    }
    /**
     * Origin URI
     */
    get baseURL() {
        const { address: { address, port } = {} } = this;
        return address ? `http://${address}:${port}` : '';
    }
    /**
     * URL to open in default browser
     */
    get openURL() {
        const { openPath, baseURL } = this;
        return typeof openPath !== 'string'
            ? baseURL
            : new URL(openPath, baseURL) + '';
    }
    static getIPA() {
        const address = internal_ip_1.v4.sync() || internal_ip_1.v6.sync() || '127.0.0.1';
        return {
            family: address.includes(':') ? 'IPv6' : 'IPv4',
            address
        };
    }
    /**
     * Create a server in the same Node.JS process
     */
    localHost() {
        const server = this;
        return this.core
            .listen(this.netPort, async function () {
            server.address = Object.assign(Object.assign({}, this.address()), WebServer.getIPA());
            if (process.send instanceof Function)
                return process.send({
                    type: 'ready',
                    data: server.address
                });
            console.info(`Web server runs at ${server.baseURL}`);
            if (server.openPath)
                await (0, open_1.default)(server.openURL + '');
        })
            .on('error', error => {
            if (process.send instanceof Function)
                process.send({
                    type: 'error',
                    data: error
                });
            else
                console.error(error);
        });
    }
    /**
     * Boot a server in a forked Node.JS process
     */
    workerHost() {
        const child = (0, child_process_1.fork)((0, path_1.join)((0, node_toolkit_1.currentModulePath)(), '../../dist/'), [
            this.staticPath,
            '-p',
            this.netPort + '',
            this.XDomain && '--CORS'
        ], { execArgv: [] });
        return new Promise((resolve, reject) => child.on('message', ({ type, data }) => {
            switch (type) {
                case 'ready':
                    return resolve(data);
                case 'error': {
                    const _a = data, { name, message } = _a, rest = __rest(_a, ["name", "message"]);
                    reject(Object.assign(new globalThis[name](message), rest));
                }
            }
        }));
    }
}
exports.WebServer = WebServer;
